@@ -5,6 +5,15 @@ from django.db.models.signals import post_save
 
 _registered_models = set([])
 _auto_filter_models = set([])
+_unrestricted_access_hooks = {}
+
+
+def _is_superuser(user):
+    return user.is_superuser
+
+
+def get_unrestricted_access_hooks(model):
+    return _unrestricted_access_hooks[model]
 
 
 def is_registered_model(model):
@@ -17,17 +26,19 @@ def is_auto_filtered(model):
 
 def register(
     model, control_relation=False, unrestricted_manager=False,
-    auto_filter=True, owner=True):
+    auto_filter=True, owner=True, unrestricted_access_hooks=[]):
     """
     Register a model with the access control code.
     """
     from django_group_access.models import (
-        AccessGroup, process_auto_share_groups)
+        get_group_model, process_auto_share_groups)
 
     if is_registered_model(model):
         return
     _registered_models.add(model)
 
+    _unrestricted_access_hooks[model] = [_is_superuser] + unrestricted_access_hooks
+
     if auto_filter:
         _auto_filter_models.add(model)
 
@@ -51,6 +62,6 @@ def register(
         # add the attribute to the class.
         return
     ManyToManyField(
-        AccessGroup, blank=True, null=True).contribute_to_class(
+        get_group_model(), blank=True, null=True).contribute_to_class(
             model, 'access_groups')
     post_save.connect(process_auto_share_groups, model)
