@@ -1,5 +1,7 @@
 # Copyright 2012 Canonical Ltd.
 import itertools
+import unittest
+import datetime
 
 from django.test import TestCase
 from django.conf import settings
@@ -11,6 +13,9 @@ from django.contrib.auth.models import AnonymousUser, User
 
 from django_group_access import middleware, wrap_getitem
 from django_group_access import registration
+from django_group_access.models import (
+    get_group_model, process_auto_share_groups, populate_sharing,
+    model_has_field)
 from django_group_access.sandbox.models import (
     AccessRestrictedModel,
     AccessRestrictedParent,
@@ -22,18 +27,74 @@ from django_group_access.sandbox.models import (
     UniqueModel,
     UniqueForm,
 )
-from django_group_access.models import AccessGroup
+
+
+def get_members(group):
+    """
+    Returns queryset of group members. Used so that the tests
+    don't have to think about the different ways of getting
+    group members depending on the configured group model.
+    """
+    if hasattr(group, 'members'):
+        return group.members.all()
+    if hasattr(group, 'user_set'):
+        return group.user_set.all()
+
+
+def add_user(group, user):
+    """
+    Adds a user to a group instance. Automatically deals with
+    the different attribute names for group membership.
+    """
+    if hasattr(group, 'members'):
+        group.members.add(user)
+    if hasattr(group, 'user_set'):
+        group.user_set.add(user)
+
+
+class UnsupportedSignalTests(TestCase):
+    """
+    Test that the functions used in the sharing signals don't crash
+    when the group model doesn't support the functionality.
+    """
+    @unittest.skipIf(
+        hasattr(get_group_model(), 'auto_share_groups'),
+        'Group model does not have auto sharing capabilities.')
+    def test_process_auto_share_groups_skips_if_not_applicable(self):
+        """
+        Running the process_auto_share_groups function with a
+        group model that does not support it does not give
+        any errors.
+        """
+        process_auto_share_groups(None, None, None)
+
+    @unittest.skipIf(
+        hasattr(get_group_model(), 'can_share_with'),
+        'Group model does not have sharing permission capabilities.')
+    def test_populate_sharing_skips_if_not_applicable(self):
+        """
+        Running the populate_sharing function with a
+        group model that does not support it does not give
+        any errors.
+        """
+        populate_sharing(None, None, None)
 
 
 class SyncingTestCase(TestCase):
+    """
+    Makes sure the sandbox app is ready to use.
+    """
     apps = ('django_group_access.sandbox',)
 
     def _pre_setup(self):
         # Add the models to the db.
         self._original_installed_apps = list(settings.INSTALLED_APPS)
+        apps = list(settings.INSTALLED_APPS)
         for app in self.apps:
-            settings.INSTALLED_APPS.append(app)
+            apps.append(app)
+        settings.INSTALLED_APPS = apps
         loading.cache.loaded = False
+        self.group_model = get_group_model()
         call_command('syncdb', interactive=False, verbosity=0, migrate=False)
         # Call the original method that does the fixtures etc.
         super(SyncingTestCase, self)._pre_setup()
@@ -45,6 +106,39 @@ class SyncingTestCase(TestCase):
         settings.INSTALLED_APPS = self._original_installed_apps
         loading.cache.loaded = False
 
+    def _load_users(self, prefix, group):
+        """
+        Load users into a group, with a prefix on their username.
+        """
+        for i in range(3):
+            user = User.objects.create(
+                username='%s%d' % (prefix, i),
+                email='%s%d@example.com' % (prefix, i))
+            add_user(group, user)
+            add_user(self.everyone, user)
+
+    def _load_owned_models(self, group):
+        """
+        Create owned models for users in a group.
+        """
+        users = get_members(group).order_by('username')
+
+        # parent class for these resources
+        parent = AccessRestrictedParent.objects.create(
+            name='%s parent record' % group.name)
+
+        # one model per user, two for the first user
+        for (idx, user) in enumerate(users):
+            model = AccessRestrictedModel.objects.create(
+                owner=user, name='%s record %d' % (group.name, idx),
+                parent=parent)
+            model.access_groups.add(group)
+
+        user = users[0]
+        model = AccessRestrictedModel.objects.create(
+            owner=user, name='%s record extra' % group.name,
+            parent=parent)
+
 
 class AccessRelationTests(SyncingTestCase):
     """
@@ -65,12 +159,12 @@ class AccessRelationTests(SyncingTestCase):
         self.group = self._create_access_group_with_one_member()
         self.project.access_groups.add(self.group)
         self.project.save()
-        self.user_with_access = self.group.members.all()[0]
+        self.user_with_access = get_members(self.group)[0]
         self.user_without_access = _create_user()
 
     def _create_access_group_with_one_member(self):
-        group = AccessGroup.objects.create(name='oem')
-        group.members.add(_create_user())
+        group = self.group_model.objects.create(name='oem')
+        add_user(group, _create_user())
         return group
 
     def test_direct_reference(self):
@@ -146,13 +240,194 @@ class AccessRelationTests(SyncingTestCase):
         self.assertEqual(set([parent]), set(records))
 
 
-class AccessGroupSharingTest(TestCase):
+class AccessTest(SyncingTestCase):
+    """
+    Test that access restrictions are applied.
+    """
+    everyone = None
+    restricted_group_a = None
+    restricted_group_b = None
 
+    def setUp(self):
+        self.everyone = self.group_model.objects.create(name='everyone')
+
+        self.public_group = self.group_model.objects.create(name='public')
+        self.restricted_group_a = self.group_model.objects.create(name='the cabal')
+        self.restricted_group_b = self.group_model.objects.create(name='the stonecutters')
+
+        self._load_users('public', self.public_group)
+        self._load_users('cabal', self.restricted_group_a)
+        self._load_users('stonecutter', self.restricted_group_b)
+
+        self._load_owned_models(self.public_group)
+        self._load_owned_models(self.restricted_group_a)
+        self._load_owned_models(self.restricted_group_b)
+
+        User.objects.create(
+            username='nogroupuser', email='nogroup@example.com')
+        self.superuser = User.objects.create(
+            username='superuser', email='super@example.com')
+        self.superuser.is_superuser = True
+        self.superuser.save()
+        self.mike = User.objects.create(
+            username='mike', email='mike@example.com')
+
+    def test_get_own_resources(self):
+        """
+        A user has access to their owned records.
+        """
+        user = get_members(
+            self.restricted_group_a).order_by('username')[0]
+        mine = AccessRestrictedModel.objects.get_for_owner(user)
+        self.assertEqual(mine.count(), 2)
+        self.assertEqual(mine[0].name, 'the cabal record 0')
+        self.assertEqual(mine[1].name, 'the cabal record extra')
+
+    def test_accessible_parent_records(self):
+        """
+        A user can access parent records of records that he owns.
+        """
+        user = get_members(
+            self.restricted_group_a).order_by('username')[0]
+        parents = AccessRestrictedParent.objects.accessible_by_user(user)
+        self.assertEqual(parents.count(), 1)
+        self.assertEqual(parents[0].name, 'the cabal parent record')
+
+    def test_can_access_owned_records_if_not_in_a_group(self):
+        """
+        A user who is not in a group can still access their owned records.
+        """
+        user = User.objects.create(
+            username='groupless', email='groupless@example.com')
+        owned = AccessRestrictedModel.objects.create(
+            name='a record', owner=user)
+
+        record = AccessRestrictedModel.objects.accessible_by_user(
+            user).get(pk=owned.id)
+        self.assertEqual(record, owned)
+
+    def test_can_see_individual_records_shared_with_my_group(self):
+        """
+        Test that individual records become visible when shared with
+        a user's group.
+        """
+        record = AccessRestrictedModel.objects.get(
+                    name='the stonecutters record 1')
+        group = self.group_model.objects.get(name='the cabal')
+        record.access_groups.add(group)
+        record.save()
+        user = get_members(group)[0]
+        available = AccessRestrictedModel.objects.accessible_by_user(user)
+        self.assertEqual(available.count(), 5)
+        self.assertEqual(available[0].name, 'the cabal record 0')
+        self.assertEqual(available[1].name, 'the cabal record 1')
+        self.assertEqual(available[2].name, 'the cabal record 2')
+        self.assertEqual(available[3].name, 'the cabal record extra')
+        self.assertEqual(available[4].name, 'the stonecutters record 1')
+
+    def test_superusers_see_all(self):
+        """
+        Superusers see all records, regardless of group membership.
+        """
+        available = AccessRestrictedModel.objects.accessible_by_user(
+            self.superuser)
+        self.assertEqual(available.count(), 12)
+        available = AccessRestrictedParent.objects.accessible_by_user(
+            self.superuser)
+        self.assertEqual(available.count(), 3)
+
+    def test_unrestricted_access_hooks_are_called(self):
+        """
+        A model can be registered with an array of functions that
+        check if the resultsets should return all data.
+        AccessRestrictedModel has been registered with a function
+        that returns True if the user's username is 'mike'
+        """
+        available = AccessRestrictedModel.objects.accessible_by_user(
+            self.mike)
+        self.assertEqual(available.count(), 12)
+
+    def test_unrestricted_access_hooks_for_control_relation(self):
+        """
+        Models registered with a control_relation should use the
+        unrestricted_access_hooks from the model the control_relation
+        points at.
+        """
+        available = AccessRestrictedParent.objects.accessible_by_user(
+            self.mike)
+        self.assertEqual(available.count(), 3)
+
+    def test_can_see_records_with_no_controlling_access_records(self):
+        """
+        If a models is defined as having a control relation, but the
+        control relation has no records to determine access, it is
+        accessible.
+        """
+        user = _create_user()
+        record = AccessRestrictedParent.objects.create(
+            name='no related records')
+        available = AccessRestrictedParent.objects.accessible_by_user(user)
+        self.assertEqual(available.count(), 1)
+        self.assertEqual(available[0], record)
+
+
+class AccessGroupCapabilitiesTest(SyncingTestCase):
+    """
+    Test for the automatic sharing capabilities that
+    django_group_access.models.AccessGroup provides.
+    """
+    everyone = None
+    public_group = None
+    restricted_group_a = None
+    restricted_group_b = None
+    supergroup = None
+
+    def setUp(self):
+        self.everyone = self.group_model.objects.create(name='everyone')
+
+        self.public_group = self.group_model.objects.create(name='public')
+        self.restricted_group_a = self.group_model.objects.create(name='the cabal')
+        self.restricted_group_b = self.group_model.objects.create(name='the stonecutters')
+
+        self.restricted_group_b.auto_share_groups.add(self.restricted_group_b)
+        self.restricted_group_a.auto_share_groups.add(self.restricted_group_a)
+        self.public_group.auto_share_groups.add(self.everyone)
+        self.public_group.auto_share_groups.add(self.public_group)
+
+        self._load_users('public', self.public_group)
+        self._load_users('cabal', self.restricted_group_a)
+        self._load_users('stonecutter', self.restricted_group_b)
+
+        self._load_owned_models(self.public_group)
+        self._load_owned_models(self.restricted_group_a)
+        self._load_owned_models(self.restricted_group_b)
+
+        self.supergroup = self.group_model.objects.create(name='supergroup', supergroup=True)
+        add_user(self.supergroup, _create_user())
+
+        su = User.objects.create(
+                username='supergroupuser', email='supergroup@example.com')
+        add_user(self.supergroup, su)
+
+        User.objects.create(
+            username='nogroupuser', email='nogroup@example.com')
+        self.superuser = User.objects.create(
+            username='superuser', email='super@example.com')
+        self.superuser.is_superuser = True
+        self.superuser.save()
+
+    @unittest.skipIf(
+        not hasattr(get_group_model(), 'auto_share_groups'),
+        'Group model does not have auto sharing capabilities.')
     def test_can_be_shared_group_is_added_to_other_sharing_lists(self):
-        AccessGroup.objects.all().delete()
-        group_a = AccessGroup(name='A', can_be_shared_with=False)
+        """
+        A group that can be shared with is automatically added to other
+        group's sharing lists when it is created.
+        """
+        self.group_model.objects.all().delete()
+        group_a = self.group_model(name='A', can_be_shared_with=False)
         group_a.save()
-        group_b = AccessGroup(name='B', can_be_shared_with=False)
+        group_b = self.group_model(name='B', can_be_shared_with=False)
         group_b.save()
 
         self.assertEqual(str(group_a.can_share_with.all()), str([group_a]))
@@ -171,88 +446,18 @@ class AccessGroupSharingTest(TestCase):
         self.assertEqual(str(group_a.can_share_with.all()), str([group_a]))
         self.assertEqual(str(group_b.can_share_with.all()), str([group_b]))
 
-
-class AccessTest(SyncingTestCase):
-    everyone = None
-    public_group = None
-    restricted_group_a = None
-    restricted_group_b = None
-    supergroup = None
-
-    def _load_users(self, prefix, group):
-        for i in range(3):
-            u = User.objects.create_user(
-                '%s%d' % (prefix, i), '%s%d@example.com' % (prefix, i), prefix)
-            group.members.add(u)
-            self.everyone.members.add(u)
-
-    def _load_owned_models(self, group):
-        users = group.members.all().order_by('username')
-
-        # parent class for these resources
-        p = AccessRestrictedParent(name='%s parent record' % group.name)
-        p.save()
-
-        # one model per user, two for the first user
-        for (idx, u) in enumerate(users):
-            m = AccessRestrictedModel(
-                owner=u, name='%s record %d' % (group.name, idx), parent=p)
-            m.save()
-
-        u = users[0]
-        m = AccessRestrictedModel(
-            owner=u, name='%s record extra' % group.name, parent=p)
-        m.save()
-
-    def setUp(self):
-        self.everyone = AccessGroup.objects.get_or_create(name='everyone')[0]
-
-        self.public_group = AccessGroup(name='public')
-        self.public_group.save()
-        self.public_group.auto_share_groups.add(self.everyone)
-        self.public_group.auto_share_groups.add(self.public_group)
-
-        self.restricted_group_a = AccessGroup(name='the cabal')
-        self.restricted_group_a.save()
-        self.restricted_group_a.auto_share_groups.add(self.restricted_group_a)
-
-        self.restricted_group_b = AccessGroup(name='the stonecutters')
-        self.restricted_group_b.save()
-        self.restricted_group_b.auto_share_groups.add(self.restricted_group_b)
-
-        self.supergroup = AccessGroup(name='supergroup', supergroup=True)
-        self.supergroup.save()
-
-        self._load_users('public', self.public_group)
-        self._load_users('cabal', self.restricted_group_a)
-        self._load_users('stonecutter', self.restricted_group_b)
-
-        self._load_owned_models(self.public_group)
-        self._load_owned_models(self.restricted_group_a)
-        self._load_owned_models(self.restricted_group_b)
-
-        su = User.objects.create_user(
-            'supergroupuser', 'supergroup@example.com', 'test')
-        self.supergroup.members.add(su)
-
-        User.objects.create_user('nogroupuser', 'nogroup@example.com', 'test')
-        self.superuser = User.objects.create_user(
-            'superuser', 'super@example.com', 'test')
-        self.superuser.is_superuser = True
-        self.superuser.save()
-
-    def test_get_own_resources(self):
-        u = self.restricted_group_a.members.all().order_by('username')[0]
-        mine = AccessRestrictedModel.objects.get_for_owner(u)
-        self.assertEqual(mine.count(), 2)
-        self.assertEqual(mine[0].name, 'the cabal record 0')
-        self.assertEqual(mine[1].name, 'the cabal record extra')
-
-    def test_accessible_by_user(self):
-        u = self.restricted_group_a.members.all().order_by('username')[0]
+    @unittest.skipIf(
+        not hasattr(get_group_model(), 'auto_share_groups'),
+        'Group model does not have auto sharing capabilities.')
+    def test_records_shared_automatically(self):
+        """
+        Saved records should be shared with the auto_share_groups on the groups
+        the creating owner is a member of.
+        """
+        u = get_members(self.restricted_group_a).order_by('username')[0]
 
         # should return all of the records owned by someone in the user's group
-        # plus all records owned by anyone in a group marked as public
+        # plus all records automatically shared
         available = AccessRestrictedModel.objects.accessible_by_user(user=u)
         self.assertEqual(available.count(), 8)
         self.assertEqual(available[0].name, 'public record 0')
@@ -276,88 +481,21 @@ class AccessTest(SyncingTestCase):
         except AccessRestrictedModel.DoesNotExist:
             pass
 
-    def test_accessible_parent_records(self):
-        u = self.restricted_group_a.members.all().order_by('username')[0]
-        parents = AccessRestrictedParent.objects.accessible_by_user(u)
-        self.assertEqual(parents.count(), 2)
-        self.assertEqual(parents[0].name, 'public parent record')
-        self.assertEqual(parents[1].name, 'the cabal parent record')
-
-    def test_members_of_supergroup_can_see_all_records(self):
-        u = self.supergroup.members.all()[0]
-        available = AccessRestrictedModel.objects.accessible_by_user(user=u)
-        self.assertEqual(available.count(), 12)
-        self.assertEqual(available[0].name, 'public record 0')
-        self.assertEqual(available[1].name, 'public record 1')
-        self.assertEqual(available[2].name, 'public record 2')
-        self.assertEqual(available[3].name, 'public record extra')
-        self.assertEqual(available[4].name, 'the cabal record 0')
-        self.assertEqual(available[5].name, 'the cabal record 1')
-        self.assertEqual(available[6].name, 'the cabal record 2')
-        self.assertEqual(available[7].name, 'the cabal record extra')
-        self.assertEqual(available[8].name, 'the stonecutters record 0')
-        self.assertEqual(available[9].name, 'the stonecutters record 1')
-        self.assertEqual(available[10].name, 'the stonecutters record 2')
-        self.assertEqual(available[11].name, 'the stonecutters record extra')
-
-        available = AccessRestrictedParent.objects.accessible_by_user(user=u)
-        self.assertEqual(available.count(), 3)
-        self.assertEqual(available[0].name, 'public parent record')
-        self.assertEqual(available[1].name, 'the cabal parent record')
-        self.assertEqual(available[2].name, 'the stonecutters parent record')
-
-    def test_can_access_owned_records_if_not_in_a_group(self):
-        u = User.objects.create_user(
-            'groupless', 'groupless@example.com', 'nogroup')
-        m = AccessRestrictedModel(name='a record', owner=u)
-        m.save()
-
-        r = AccessRestrictedModel.objects.accessible_by_user(u).get(pk=m.id)
-        self.assertEqual(m, r)
-
-    def test_can_see_individual_records_shared_with_my_group(self):
-        record = AccessRestrictedModel.objects.get(
-                    name='the stonecutters record 1')
-        g = AccessGroup.objects.get(name='the cabal')
-        record.access_groups.add(g)
-        record.save()
-        u = g.members.all()[0]
-        available = AccessRestrictedModel.objects.accessible_by_user(u)
-        self.assertEqual(available.count(), 9)
-        self.assertEqual(available[0].name, 'public record 0')
-        self.assertEqual(available[1].name, 'public record 1')
-        self.assertEqual(available[2].name, 'public record 2')
-        self.assertEqual(available[3].name, 'public record extra')
-        self.assertEqual(available[4].name, 'the cabal record 0')
-        self.assertEqual(available[5].name, 'the cabal record 1')
-        self.assertEqual(available[6].name, 'the cabal record 2')
-        self.assertEqual(available[7].name, 'the cabal record extra')
-        self.assertEqual(available[8].name, 'the stonecutters record 1')
-
-    def test_superusers_see_all(self):
+    @unittest.skipIf(
+        not model_has_field(get_group_model(), 'supergroup'),
+        'Group model does not have supergroup capabilities.')
+    def test_members_of_supergroups_see_all(self):
         """
-        Superusers see all records, regardless of group membership.
+        Members of supergroups see all records.
         """
+        supergroup_member = get_members(self.supergroup)[0]
         available = AccessRestrictedModel.objects.accessible_by_user(
-            self.superuser)
+            supergroup_member)
         self.assertEqual(available.count(), 12)
         available = AccessRestrictedParent.objects.accessible_by_user(
-            self.superuser)
+            supergroup_member)
         self.assertEqual(available.count(), 3)
 
-    def test_can_see_records_with_no_controlling_access_records(self):
-        """
-        If a models is defined as having a control relation, but the
-        control relation has no records to determine access, it is
-        accessible.
-        """
-        user = _create_user()
-        record = AccessRestrictedParent.objects.create(
-            name='no related records')
-        available = AccessRestrictedParent.objects.accessible_by_user(user)
-        self.assertEqual(available.count(), 1)
-        self.assertEqual(available[0], record)
-
 
 class MetaInformationPropagationTest(SyncingTestCase):
     """
@@ -366,8 +504,8 @@ class MetaInformationPropagationTest(SyncingTestCase):
     and querysets correctly.
     """
     def setUp(self):
-        self.group1 = AccessGroup.objects.create(name='group1')
-        self.group2 = AccessGroup.objects.create(name='group2')
+        self.group1 = self.group_model.objects.create(name='group1')
+        self.group2 = self.group_model.objects.create(name='group2')
         self.user = _create_user()
         other_user = _create_user()
         unrestricted = Unrestricted.objects.create(
@@ -532,12 +670,12 @@ class AccessManagerMixinTest(TestCase):
 
 class RelatedRecordFilteringTest(SyncingTestCase):
     def setUp(self):
-        self.group1 = AccessGroup.objects.create(name='group1')
-        self.group2 = AccessGroup.objects.create(name='group2')
+        self.group1 = self.group_model.objects.create(name='group1')
+        self.group2 = self.group_model.objects.create(name='group2')
         self.user = _create_user()
         other_user = _create_user()
-        self.group1.members.add(self.user)
-        self.group2.members.add(other_user)
+        add_user(self.group1, self.user)
+        add_user(self.group2, other_user)
         self.parent1 = AccessRestrictedParent.objects.create(name='parent1')
         self.parent2 = AccessRestrictedParent.objects.create(name='parent2')
         self.child1 = AccessRestrictedModel.objects.create(
@@ -579,8 +717,8 @@ class RelatedRecordFilteringTest(SyncingTestCase):
 
 class ManyToManyRelatedRecordFilteringTest(SyncingTestCase):
     def setUp(self):
-        self.group1 = AccessGroup.objects.create(name='group1')
-        self.group2 = AccessGroup.objects.create(name='group2')
+        self.group1 = self.group_model.objects.create(name='group1')
+        self.group2 = self.group_model.objects.create(name='group2')
         self.user = _create_user()
         other_user = _create_user()
         self.project1 = Project.objects.create(
@@ -767,8 +905,8 @@ class UnrestrictedAccessTest(SyncingTestCase):
     calling .unrestricted()
     """
     def setUp(self):
-        self.group1 = AccessGroup.objects.create(name='group1')
-        self.group2 = AccessGroup.objects.create(name='group2')
+        self.group1 = self.group_model.objects.create(name='group1')
+        self.group2 = self.group_model.objects.create(name='group2')
         self.user = _create_user()
         other_user = _create_user()
         self.project1 = Project.objects.create(
@@ -802,8 +940,8 @@ class DbMethodsHaveRestrictionsAppliedTest(SyncingTestCase):
     has been filtered with accessible_by_user.
     """
     def setUp(self):
-        self.group1 = AccessGroup.objects.create(name='group1')
-        self.group2 = AccessGroup.objects.create(name='group2')
+        self.group1 = self.group_model.objects.create(name='group1')
+        self.group2 = self.group_model.objects.create(name='group2')
         self.user = _create_user()
         self.other_user = _create_user()
         self.project1 = Project.objects.create(
@@ -874,8 +1012,8 @@ class RefactorBugsTest(SyncingTestCase):
     support .unrestricted() on querysets.
     """
     def setUp(self):
-        self.group1 = AccessGroup.objects.create(name='group1')
-        self.group2 = AccessGroup.objects.create(name='group2')
+        self.group1 = self.group_model.objects.create(name='group1')
+        self.group2 = self.group_model.objects.create(name='group2')
         self.user = _create_user()
         self.other_user = _create_user()
         self.project1 = Project.objects.create(
@@ -947,8 +1085,8 @@ class AutomaticFilteringTest(SyncingTestCase):
         class MockRequest(object):
             pass
 
-        self.group1 = AccessGroup.objects.create(name='group1')
-        self.group2 = AccessGroup.objects.create(name='group2')
+        self.group1 = self.group_model.objects.create(name='group1')
+        self.group2 = self.group_model.objects.create(name='group2')
         self.user = _create_user()
         self.other_user = _create_user()
         self.project1 = Project.objects.create(
@@ -1205,11 +1343,11 @@ class UniqueContraintsInModelForms(SyncingTestCase):
             pass
         self.MockRequest = MockRequest
         self.user = _create_user()
-        group1 = AccessGroup.objects.create(name='group1')
-        group2 = AccessGroup.objects.create(name='group2')
+        group1 = self.group_model.objects.create(name='group1')
+        group2 = self.group_model.objects.create(name='group2')
         other_user = _create_user()
-        group1.members.add(self.user)
-        group2.members.add(other_user)
+        add_user(group1, self.user)
+        add_user(group2, other_user)
         # self.user cannot see this record
         record = UniqueModel.objects.create(name='my name')
         record.access_groups.add(group2)
@@ -1235,11 +1373,29 @@ class UniqueContraintsInModelForms(SyncingTestCase):
             'were used to check uniqueness.')
 
 
+class SubqueryTest(SyncingTestCase):
+    def test_querysets_used_in_subqueries(self):
+        """
+        An access restricted queryset used inside another queryset
+        should have the access controls applied.
+        """
+        user = User.objects.create(username='user')
+        group = self.group_model.objects.create(name='group')
+        project = Project.objects.create(name='project')
+        project.access_groups.add(group)
+        subqueryset = Project.objects.all().accessible_by_user(user)
+
+        # there should be no access groups associated with the list
+        # of projects in the subqueryset
+        groups = self.group_model.objects.filter(project__in=subqueryset)
+        self.assertEqual(set(groups), set([]))
+
+
 counter = itertools.count()
 
 
 def _create_user():
     random_string = 'asdfg%d' % counter.next()
-    user = User.objects.create_user(
-        random_string, '%s@example.com' % random_string)
+    user = User.objects.create(
+        username=random_string, email='%s@example.com' % random_string)
     return user
